"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generator_helper_1 = require("@prisma/generator-helper");
const internals_1 = require("@prisma/internals");
const lodash_1 = __importDefault(require("lodash"));
const constants_1 = require("./constants");
const writeFileSafely_1 = require("./utils/writeFileSafely");
const { version } = require('../package.json');
function emitType(type) {
    switch (type) {
        case 'Int':
            return 'Integer';
        case 'Json':
            return 'JSON';
        case 'Bytes':
            return 'LargeBinary';
        case 'Float':
            return 'Float';
        case 'BigInt':
            return 'BigInteger';
        default:
            return type;
    }
}
function emitPythonType(field) {
    let type = field.type;
    switch (type) {
        case 'Int':
            type = 'int';
            break;
        case 'Json':
            type = 'Any';
            break;
        case 'Bytes':
            type = 'bytes';
            break;
        case 'Float':
            type = 'float';
            break;
        case 'String':
            type = 'str';
            break;
        case 'Boolean':
            type = 'bool';
            break;
        case 'DateTime':
            type = 'datetime';
            break;
        case 'BigInt':
            type = 'BigInteger';
            break;
        default:
            type = type;
            break;
    }
    if (!field.isRequired) {
        type = `Optional[${type}]`;
    }
    return type;
}
function isRelationshipField(field) {
    return !!field.relationToFields || !!field.relationFromFields;
}
function attributeName(model) {
    if (model.name == 'createdAt') {
        return 'created_at';
    }
    else if (model.name == 'updatedAt') {
        return 'updated_at';
    }
    else {
        return model.name;
    }
}
function emitField(model) {
    if (model.name == 'createdAt' ||
        model.name == 'created_at' ||
        model.name == 'updated_at' ||
        model.name == 'updatedAt')
        return undefined;
    if (isRelationshipField(model)) {
        if (model.relationFromFields && model.relationToFields) {
            return ('# ' +
                model.name +
                ': ' +
                model.relationFromFields[0] +
                ' -> ' +
                model.relationToFields[0]);
        }
        else {
            return undefined;
        }
    }
    const args = lodash_1.default.compact([
        emitType(model.type),
        model.isId ? 'primary_key=True' : null,
        !model.isRequired ? 'nullable=True' : null,
        model.isUnique ? 'unique=True' : null,
        model.relationToFields,
        `name="${model.name}"`,
    ]);
    return `\t${attributeName(model)}: ${emitPythonType(model)} = Column(${args.join(', ')})`;
}
function indent(str) {
    return str
        .split('\n')
        .map((l) => '\t' + l)
        .join('\n');
}
function isRequiredArgument(f) {
    return (f.isRequired &&
        !f.hasDefaultValue &&
        !f.isUpdatedAt &&
        !isMagicTimestampField(f));
}
function defaultArg(f) {
    if (!isRequiredArgument(f)) {
        return '=None';
    }
    return '';
}
function emitConstructor(fields) {
    fields = fields.filter((f) => !isRelationshipField(f));
    const argList = indent('self,\n' +
        fields
            .sort((a, b) => {
            const x = isRequiredArgument(a);
            const y = isRequiredArgument(b);
            return x == y ? 0 : x > y ? -1 : 1;
        })
            .map((f) => `${attributeName(f)}: ${emitPythonType(f)}${defaultArg(f)}`)
            .join(',\n'));
    const superInvocation = indent(fields.map((f) => `${attributeName(f)}=${attributeName(f)}`).join(',\n'));
    const body = indent(`super().__init__(\n${superInvocation}\n)`);
    return `
def __init__(\n${argList}\n):
${body}
`;
}
function emitModel(model) {
    const fields = lodash_1.default.compact(model.fields.map((f) => emitField(f))).join('\n');
    let docstring = '\t';
    if (model.documentation) {
        docstring = `\t""" ${model.documentation}.\nDefined in frontend/prisma/schema.prisma """`;
    }
    const tablename = model.dbName || model.name;
    const generatedConstructor = emitConstructor(model.fields);
    return `
class ${model.name}SQL(Base):
${docstring}
\t__tablename__ = "${tablename}"

${indent(generatedConstructor)}

${fields}
`;
}
function emitHeader() {
    return `
from sqlalchemy import Column, String, Integer, Boolean, DateTime, LargeBinary, Float, BigInteger
from sqlalchemy.dialects.postgresql import JSON
from server.utils.sql.session import Base

from datetime import datetime
from typing import Any, Optional
`;
}
(0, generator_helper_1.generatorHandler)({
    onManifest() {
        internals_1.logger.info(`${constants_1.GENERATOR_NAME}:Registered`);
        internals_1.logger.info('Registered!!');
        return {
            version,
            defaultOutput: '../generated',
            prettyName: constants_1.GENERATOR_NAME,
        };
    },
    onGenerate: async (options) => {
        var _a;
        const header = emitHeader();
        const output = header +
            options.dmmf.datamodel.models
                .map((m) => emitModel(m))
                .join('\n\n')
                .replace(/\t/g, '    ');
        const writeLocation = (_a = options.generator.output) === null || _a === void 0 ? void 0 : _a.value;
        internals_1.logger.info('Writing to ', writeLocation);
        await (0, writeFileSafely_1.writeFileSafely)(writeLocation, output);
    },
});
function isMagicTimestampField(f) {
    return f.name == 'createdAt' || !!f.isUpdatedAt || f.name == 'updatedAt';
}
//# sourceMappingURL=generator.js.map